"use strict";

exports.__esModule = true;
exports.default = chunkNameProperty;

var _vm = _interopRequireDefault(require("vm"));

var _util = require("../util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var WEBPACK_CHUNK_NAME_REGEXP = /webpackChunkName/;

function readWebpackChunkNameValue(value) {
  // try compile only if webpack options comment is present
  try {
    var val = _vm.default.runInNewContext("(function(){return {" + value + "};})()");

    return val.webpackChunkName;
  } catch (e) {
    throw Error("compilation error while processing: /*" + value + "*/: " + e.message);
  }
}

function getChunkNameComment(importArg) {
  if (!importArg.has('leadingComments')) return null;
  return importArg.get('leadingComments').find(function (comment) {
    return comment.node.value.match(WEBPACK_CHUNK_NAME_REGEXP);
  });
}

function getRawChunkNameFromCommments(importArg) {
  var chunkNameComment = getChunkNameComment(importArg);
  if (!chunkNameComment) return null;
  return readWebpackChunkNameValue(chunkNameComment.node.value);
}

function moduleToChunk(str) {
  return str ? str.replace(/^[./]+|(\.js$)/g, '').replace(/\//, '-') : '';
}

function replaceQuasiValue(str) {
  return str ? str.replace(/\//g, '-') : str;
}

function chunkNameProperty(_ref) {
  var t = _ref.types;

  function transformQuasi(quasi, index) {
    if (index === 0) {
      return t.templateElement({
        raw: moduleToChunk(quasi.value.raw),
        cooked: moduleToChunk(quasi.value.cooked)
      }, quasi.tail);
    }

    return t.templateElement({
      raw: replaceQuasiValue(quasi.value.raw),
      cooked: replaceQuasiValue(quasi.value.cooked)
    }, quasi.tail);
  }

  function generateChunkName(callPath) {
    var importArg = (0, _util.getImportArg)(callPath);

    if (importArg.isTemplateLiteral()) {
      return t.templateLiteral(importArg.node.quasis.map(transformQuasi), importArg.node.expressions);
    }

    return t.stringLiteral(moduleToChunk(importArg.node.value));
  }

  function getExistingChunkName(callPath) {
    var importArg = (0, _util.getImportArg)(callPath);
    var chunkName = getRawChunkNameFromCommments(importArg);
    if (!chunkName) return null;
    return t.stringLiteral(chunkName);
  }

  function isAgressiveImport(callPath) {
    var importArg = (0, _util.getImportArg)(callPath);
    return importArg.isTemplateLiteral() && importArg.node.expressions.length > 0;
  }

  function addChunkNameComment(callPath, chunkName) {
    var importArg = (0, _util.getImportArg)(callPath);
    var chunkNameComment = getChunkNameComment(importArg);

    if (chunkNameComment) {
      chunkNameComment.remove();
    }

    if (isAgressiveImport(callPath)) {
      importArg.addComment('leading', ' webpackChunkName: "[request]" ');
      return;
    }

    var value = t.isTemplateLiteral(chunkName) ? chunkName.quasis[0].value.cooked : chunkName.value;
    importArg.addComment('leading', " webpackChunkName: \"" + value + "\" ");
  }

  return function (_ref2) {
    var callPath = _ref2.callPath,
        funcPath = _ref2.funcPath;
    var chunkName;
    var agressiveImport = isAgressiveImport(callPath);

    if (!agressiveImport) {
      chunkName = getExistingChunkName(callPath);
    }

    if (!chunkName) {
      chunkName = generateChunkName(callPath);
      addChunkNameComment(callPath, chunkName);
    }

    return t.objectMethod('method', t.identifier('chunkName'), funcPath.node.params, t.blockStatement([t.returnStatement(chunkName)]));
  };
}